"""Tests the schema that is dynamically generated from the configuration"""
import pytest
from singer_sdk.tap_base import Tap

from tap_feed.tap import TapFeed


@pytest.mark.parametrize(
    "configuration,expected_schema_keys",
    [
        (
            {
                "feed_urls": [
                    "http://feeds.feedburner.com/PythonSoftwareFoundationNews"
                ],
                "feed_fields": ["title", "subtitle"],
                "feed_entry_fields": ["id", "title", "summary", "link"],
            },
            [
                "feed_url",
                "feed_title",
                "feed_subtitle",
                "entry_id",
                "entry_published",
                "entry_title",
                "entry_summary",
                "entry_link",
            ],
        ),
        (
            {
                "feed_urls": [
                    "http://feeds.feedburner.com/PythonSoftwareFoundationNews"
                ],
                "feed_fields": ["title", "subtitle"],
                "feed_entry_fields": ["published", "title", "summary", "link"],
            },
            [
                "feed_url",
                "feed_title",
                "feed_subtitle",
                "entry_id",
                "entry_published",
                "entry_title",
                "entry_summary",
                "entry_link",
            ],
        ),
        (
            {
                "feed_urls": [
                    "http://feeds.feedburner.com/PythonSoftwareFoundationNews"
                ],
                "feed_fields": ["title"],
                "feed_entry_fields": ["title", "summary"],
            },
            [
                "feed_url",
                "feed_title",
                "entry_id",
                "entry_published",
                "entry_title",
                "entry_summary",
            ],
        ),
        (
            {
                "feed_urls": [
                    "http://feeds.feedburner.com/PythonSoftwareFoundationNews"
                ],
                "feed_fields": ["language", "title"],
                "feed_entry_fields": ["title", "episode", "description"],
            },
            [
                "feed_url",
                "feed_language",
                "feed_title",
                "entry_id",
                "entry_published",
                "entry_title",
                "entry_episode",
                "entry_description",
            ],
        ),
        (
            {
                "feed_urls": [
                    "http://feeds.feedburner.com/PythonSoftwareFoundationNews"
                ],
                "feed_fields": ["language", "title"],
                "feed_entry_fields": ["title", "episode", "description"],
                "feed_entry_replication_key": "updated",
            },
            [
                "feed_url",
                "feed_language",
                "feed_title",
                "entry_id",
                "entry_updated",
                "entry_title",
                "entry_episode",
                "entry_description",
            ],
        ),
    ],
)
def test_schema_generated(configuration, expected_schema_keys):
    """Verifies the schema is generated correctly from the configuration provided"""
    test_tap: Tap = TapFeed(config=configuration)
    test_tap.run_discovery()
    catalog = test_tap.catalog_dict
    assert (
        list(catalog["streams"][0]["schema"]["properties"].keys())
        == expected_schema_keys
    )



"""Tests tap configuration that is used in other parts of the code base"""

import pytest
from singer_sdk.tap_base import Tap

from tap_feed.tap import TapFeed

SAMPLE_CONFIG = {
    "feed_urls": ["http://feeds.feedburner.com/PythonSoftwareFoundationNews"],
}


class TestConfigJsonSchema:
    """Test suite verifying the config_jsonschema attribute"""

    def test_property_names(self):
        """Verify the property names are the values used elsewhere in the code base"""
        test_tap: Tap = TapFeed(config=SAMPLE_CONFIG)
        expected_properties = [
            "feed_urls",
            "feed_fields",
            "feed_entry_fields",
            "feed_entry_replication_key",
            "start_date",
        ]
        assert (
            list(test_tap.config_jsonschema["properties"].keys()) == expected_properties
        )

    def test_required_field_feed_urls(self):
        """Verifies an exception is thrown if the feed_urls property is not provided"""
        with pytest.raises(RuntimeError) as excinfo:
            test_tap: Tap = TapFeed()
            print(test_tap.config_jsonschema)

        assert "'feed_urls' is a required property" in str(excinfo.value)

    @pytest.mark.parametrize(
        "property_name,expected_default",
        [
            ("feed_fields", ["title"]),
            ("feed_entry_fields", ["id", "title", "link"]),
        ],
    )
    def test_default_values(self, property_name, expected_default):
        """Verifies the default values for the json schema properties"""
        test_tap: Tap = TapFeed(config=SAMPLE_CONFIG)
        assert (
            test_tap.config_jsonschema["properties"][property_name]["default"]
            == expected_default
        )


def test_stream_discovery():
    """Verifies the correct stream is discovered by the tap"""
    test_tap: Tap = TapFeed(config=SAMPLE_CONFIG)
    streams_discovered = test_tap.discover_streams()
    assert len(streams_discovered) == 1
    assert streams_discovered[0].name == "feed"



"""Feed tap class."""

from typing import List

from singer_sdk import Tap, Stream
from singer_sdk import typing as th  # JSON Schema typing helpers

from tap_feed.streams import (
    FeedStream,
)

STREAM_TYPES = [
    FeedStream,
]


class TapFeed(Tap):
    """TapFeed class."""

    name = "tap-feed"

    config_jsonschema = th.PropertiesList(
        th.Property(
            "feed_urls",
            th.ArrayType(th.StringType),
            required=True,
            description="A list of one or more feed paths.",
        ),
        th.Property(
            "feed_fields",
            th.ArrayType(th.StringType),
            required=True,
            default=["title"],
            description="A list of feed level data fields to capture.",
        ),
        th.Property(
            "feed_entry_fields",
            th.ArrayType(th.StringType),
            required=True,
            default=["id", "title", "link"],
            description="A list of entry level data fields to capture.",
        ),
        th.Property(
            "feed_entry_replication_key",
            th.StringType,
            required=True,
            default="published",
            description="The field used to determine new records, typically 'published' or 'updated'.",
        ),
        th.Property(
            "start_date",
            th.DateTimeType,
            description="The earliest record date to sync",
        ),
    ).to_dict()

    def discover_streams(self) -> List[Stream]:
        """Return a list of discovered streams."""
        FeedStream.replication_key = (
            f"entry_{self.config['feed_entry_replication_key']}"
        )
        return [FeedStream(tap=self)]



"""Tests standard tap features using the built-in SDK tests library."""

import datetime

from singer_sdk.testing import get_standard_tap_tests

from tap_feed.tap import TapFeed

SAMPLE_CONFIG = {
    "feed_urls": ["http://feeds.feedburner.com/PythonSoftwareFoundationNews"],
    "start_date": datetime.datetime.now(datetime.timezone.utc).strftime("%Y-%m-%d"),
}


# Run standard built-in tap tests from the SDK:
def test_standard_tap_tests():
    """Run standard tap tests from the SDK."""
    tests = get_standard_tap_tests(TapFeed, config=SAMPLE_CONFIG)
    for test in tests:
        test()



"""Tests parsing of Atom 2.0 feed."""

from datetime import timedelta
import json
from typing import List

import pytest
from singer_sdk.tap_base import Tap

from tap_feed.tap import TapFeed


mock_feed_text = """
This XML file does not appear to have any style information associated with it. The document tree is shown below.
<feed xmlns="http://www.w3.org/2005/Atom">
<title>
<![CDATA[ Global Hunger Index (GHI) - peer-reviewed annual publication designed to comprehensively measure and track hunger at the global, regional, and country levels ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org"/>
<subtitle type="html">
<![CDATA[ The Global Hunger Index is a peer-reviewed annual report, jointly published by Concern Worldwide and Welthungerhilfe, designed to comprehensively measure and track hunger at the global, regional, and country levels. The aim of the GHI is to trigger action to reduce hunger around the world. ]]>
</subtitle>
<rights>
<![CDATA[ Copyright 2021, Global Hunger Index (GHI) - peer-reviewed annual publication designed to comprehensively measure and track hunger at the global, regional, and country levels ]]>
</rights>
<updated>2021-10-19T01:28:10+00:00</updated>
<id>urn:uuid:12a48ca2-c77f-568c-f307-0a993066692a</id>
<generator uri="http://www.elxis.org/" version="5.2">Elxis</generator>
<author>
<name>
<![CDATA[ Ethical Sector ]]>
</name>
</author>
<icon>https://www.globalhungerindex.org/media/images/favicon.ico</icon>
<logo>https://www.globalhungerindex.org/media/images/logo_rss.png</logo>
<entry>
<title>
<![CDATA[ Global Hunger Index Scores by 2021 GHI Rank ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/ranking.html"/>
<id>urn:uuid:0e8469c6-d051-2bfe-d50e-befebb7913dc</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles4/mockup-poster_thumb.jpg" alt="Global Hunger Index Scores by 2021 GHI Rank" /> <strong>For the 2021 GHI report, data were assessed for 135 countries. Out of these, there were sufficient data to calculate 2021 GHI scores for and rank 116 countries.</strong><br /> ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles4/mockup-poster_thumb.jpg" length="5553" type="image/jpeg"/>
</entry>
<entry>
<title>
<![CDATA[ Download the Global Hunger Index ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/download/all.html"/>
<id>urn:uuid:e2b4457b-f0a2-236c-ca99-3561c1e065bc</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles/mockup_thumb.jpg" alt="Download the Global Hunger Index" /> <strong>The Global Hunger Index was first produced in 2006. It is published every October. The 2021 edition marks the 16th edition of the GHI.</strong><br /> ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles/mockup_thumb.jpg" length="4174" type="image/jpeg"/>
</entry>
<entry>
<title>
<![CDATA[ Hunger and Food Systems in Conflict Settings ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/issues-in-focus/2021.html"/>
<id>urn:uuid:455fe9b7-f932-e476-b2cd-9d3c8d165394</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles4/essay_thumb.jpg" alt="Hunger and Food Systems in Conflict Settings" /> <strong>Failing food systems and the consequent increase in hunger are among the most pressing issues of our time. The world is falling far short of what is needed to achieve Zero Hunger—the second of the United Nations’ Sustainable Development Goals (SDGs).</strong><br /> Hunger and Food Systems in Conflict Settings &nbsp;&nbsp;&nbsp; By Caroline Delgado and Dan Smith Stockholm International Peace Research Institute October 2021 About image Photo: Welthungerhilfe/Stefanie Glinski 2018; At a village market in South Sudan, a woman sells fruits and vegetables to earn her livelihood. By boosting livelihood security, resilient food systems contribute to peace building. Thus, especially in conflict-affected contexts, local markets play an important role in the recovery of the households of both vendors and consumers. Hide ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles4/essay_thumb.jpg" length="8512" type="image/jpeg"/>
</entry>
<entry>
<title>
<![CDATA[ Policy Recommendations ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/policy-recommendations.html"/>
<id>urn:uuid:1113eed6-c6b5-bbfd-62fa-51c0cd0560be</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles4/policy-recommendations_thumb.jpg" alt="Policy Recommendations" /> <strong>Concern Worldwide and Welthungerhilfe share a mission to eradicate hunger, and produce the GHI every year to track hunger levels around the world, understand progress, and spotlight areas for action.</strong><br /> Policy Recommendations &nbsp; &nbsp; &nbsp;&nbsp;&nbsp; About image Photo: Concern Worldwide/Ollivier Girard 2021; A woman waters vegetables in the communal garden in the village of Toungaïli, Tahoua region, Niger. Climate volatility and conflict directly affect the agricultural livelihoods of thousands of communities. Resilient and climate-smart agriculture is, therefore, key to improving food and nutrition security. Hide header ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles4/policy-recommendations_thumb.jpg" length="9344" type="image/jpeg"/>
</entry>
<entry>
<title>
<![CDATA[ Global, Regional, and National Trends ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/trends.html"/>
<id>urn:uuid:f7dddddb-4664-2c63-de2d-9ee26cc26987</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles4/map_thumb.png" alt="Global, Regional, and National Trends" /> <strong>The fight against hunger is dangerously off track. Based on current GHI projections, the world as a whole - and 47 countries in particular - will fail to achieve a low level of hunger by 2030.</strong><br /> Global, Regional and National Trends &nbsp; &nbsp; &nbsp;&nbsp;&nbsp; About image Photo: Welthungerhilfe 2021; Puja Jatav sieves edible grains at a Nutrition Smart Village, a relief initiative that fosters nutrition-sensitive agriculture, in the village of Haripur, India. People have been severely hit by COVID-19 and by pandemic-related restrictions in India, the country with the highest child wasting rate worldwide. Hide Key Messages The fight against hunger is dangerously off track. Based on current GHI projections, the world as a whole - and 47 countries in particular - will fail to achieve a low level of hunger by 2030. Food security is under assault on multiple fronts. Worsening conflict, weather extremes associated with global climate change, and the economic and health challenges associated with the COVID-19 pandemic are all driving hunger. After decades of decline, the global prevalence of undernourishment - a component of the Global Hunger Index - is increasing. This shift may be a leading indicator of reversals in other measures of hunger. Africa South of the Sahara and South Asia are the world regions where hunger levels are highest. Hunger in both regions is considered serious. Dozens of countries suffer from severe hunger. According to the 2021 GHI scores and provisional designations, drawing on data from 2016–2020, hunger is considered extremely alarming in one country (Somalia), alarming in 9 countries, and serious in 37 countries. Inequality - between regions, countries, districts, and communities - is pervasive and, left unchecked, will keep the world from achieving the Sustainable Development Goal (SDG) mandate to “leave no one behind.” It is difficult to be optimistic about hunger in 2021. The forces now driving hunger are overpowering good intentions and lofty goals. Among the most powerful and toxic of these forces are conflict, climate change, and COVID-19—three Cs that threaten to wipe out any progress that has been made against hunger in recent years. Violent conflict, which is deeply intertwined with hunger, shows no signs of abating. The consequences of climate change are becoming ever more apparent (Masson-Delmotte et al. 2021) and costly, but the world has developed no fully effective mechanism to slow, much less reverse, it (Raiser et al. 2020). And the COVID-19 pandemic, which has spiked in different parts of the world throughout 2020 and 2021, has shown just how vulnerable we are to global contagion and the associated health and economic consequences. As we struggle to contain the current pandemic, we must be realistic that this will not be the last. As a result of these forces—as well as a host of underlying factors such as poverty, inequality, unsustainable food systems, lack of investment in agriculture and rural development, inadequate safety nets, and poor governance—progress in the fight against hunger shows signs of stalling and even being reversed. It is in this dire context that the hunger situation is playing out in the world as a whole, in global regions, and in individual countries. Figure 1.1 WORLD GHI SCORES AND PREVALENCE OF UNDERNOURISHMENT IN RECENT DECADES Note: GHI scores for the year 2000 include data from 1998–2002; 2006 GHI scores include data from 2004–2008; 2012 GHI scores include data from 2010–2014; and 2021 GHI scores include data from 2016–2020. Data on undernourishment are from FAO (2021). The undernourishment values include data from high-income countries with low levels of hunger, which are excluded from the GHI. For a complete list of data sources for the calculation of GHI scores, see Appendix C. ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles4/map_thumb.png" length="26903" type="image/png"/>
</entry>
<entry>
<title>
<![CDATA[ Assessing the Severity of Hunger in Countries with Incomplete Data ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/designations.html"/>
<id>urn:uuid:7b5ceafb-f3f8-93fc-148f-1d2adbfa5e61</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles2/2020-box-1-3_thumb.png" alt="Assessing the Severity of Hunger in Countries with Incomplete Data" /> <strong>In this year’s GHI report, 19 countries that met the criteria for inclusion in the GHI had insufficient data to allow for calculation of a 2021 GHI score.</strong><br /> Box 1.3 ASSESSING THE SEVERITY OF HUNGER IN COUNTRIES WITH INCOMPLETE DATA In this year’s GHI report, 19 countries that met the criteria for inclusion in the GHI had insufficient data to allow for calculation of a 2021 GHI score. To address this gap and give a preliminary picture of hunger in the countries with missing data, the table below indicates provisional designations of the severity of hunger. These designations are based on those GHI indicator values that are available, the country’s last known GHI severity designation, the country’s last known prevalence of undernourishment, the prevalence of undernourishment for the subregion in which the country is located, and/or an examination of the 2019, 2020, and 2021 editions of the Global Report on Food Crises (FSIN 2019; FSIN and GNAFC 2020, 2021). In some cases, data are missing because of violent conflict or political unrest (FAO, IFAD et al. 2017; Martin-Shields and Stojetz 2019), which are strong predictors of hunger and undernutrition (see Box 1.4 and this year's essay). The countries with missing data may often be those facing the greatest hunger burdens. Of the 4 countries provisionally designated as alarming - Burundi, Comoros, South Sudan, and Syrian Arab Republic - it is possible that with complete data, one or more of them would fall into the extremely alarming category. However, without sufficient information to confirm that this is the case, we have conservatively categorized each of these countries as alarming. ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles2/2020-box-1-3_thumb.png" length="16096" type="image/png"/>
</entry>
<entry>
<title>
<![CDATA[ About: The Concept of the Global Hunger Index ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/about.html"/>
<id>urn:uuid:6d57252a-8bcf-f2f6-2475-77ae98afc0d3</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles4/about_thumb.jpg" alt="About: The Concept of the Global Hunger Index" /> <strong>The Global Hunger Index (GHI) is a tool designed to comprehensively measure and track hunger at global, regional, and national levels. GHI scores are calculated each year to assess progress and setbacks in combating hunger.</strong><br /> About &nbsp; The Concept of the Global Hunger Index &nbsp; &nbsp;&nbsp;&nbsp; About image Photo: Welthungerhilfe/Papa Shabani 2021; A smallholder farmer sells onions at a market in Luweero, Uganda. In many areas, disruptions to food systems triggered by the COVID-19 pandemic have undermined the livelihoods of small-scale farmers. Building resilient food systems requires not only raising agricultural productivity but also strengthening food transport, storage, and distribution. Hide ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles4/about_thumb.jpg" length="8346" type="image/jpeg"/>
</entry>
<entry>
<title>
<![CDATA[ Partner Spotlight: Welthungerhilfe In Sudan ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/case-studies/2021-sudan.html"/>
<id>urn:uuid:573688ed-7c4b-db7a-d575-8f71efa674f8</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles4/article355_thumb.png" alt="Partner Spotlight: Welthungerhilfe In Sudan" /> <strong>Welthungerhilfe, with its mandate to work on both humanitarian assistance and development cooperation, operates in many countries affected by violent conflict. One such country is Sudan.</strong><br /> Box 2.1 PARTNER SPOTLIGHT: WELTHUNGERHILFE IN SUDAN Welthungerhilfe, with its mandate to work on both humanitarian assistance and development cooperation, operates in many countries affected by violent conflict. One such country is Sudan, where decades of conflict, coupled with economic downturns, have led to widespread hunger. With a 2021 GHI score of 25.1, Sudan suffers from a serious level of hunger and ranks 95th out of 116 countries. A record 9.8 million people in Sudan—one-fifth of the population analyzed—faced high projected levels of acute food insecurity between June and September 2021 and require urgent assistance. North Darfur is forecast to be the worst-affected area (IPC 2021b). Operating in Sudan requires a clear understanding of the historical causes of conflict and its drivers, which are complex, politicized, and multi-level, encompassing local, national, regional, and international dimensions at the same time. The country has large numbers of both internally displaced persons (IDPs) and refugees from neighboring countries (IOM and WFP 2021; UNHCR 2021). Tensions over scarce livelihood assets and land have arisen between host communities and displaced persons as well as between pastoralists and farmers, particularly along migratory routes. Droughts, desertification, and floods are contributing to new conflicts in an environment where resources and opportunities are already under stress (OCHA 2020). It is now widely recognized that there can be no food and nutrition security without peace. To strengthen resilience and achieve food and nutrition security, Welthungerhilfe strives to take a systemic approach to food systems, including in conflict settings such as Sudan. It works along the humanitarian–development– peace-building nexus to provide relief and recovery in the event of acute shocks and stresses while strengthening resilience and livelihoods for host communities, IDPs, and refugees. Placing communities at the center of its work, Welthungerhilfe’s program also supports community-level peace-building initiatives. North Darfur is the region of focus for Welthungerhilfe’s operations in Sudan, along with the states of Gedaref, Kassala, and Red Sea. Welthungerhilfe addresses the most critical humanitarian needs of host communities, IDPs, and refugees through cash and voucher assistance, protection, shelter, nonfood items, and water, sanitation, and hygiene. It links those interventions with others aimed at improving human security, resilience, food and nutrition security, and livelihoods, as well as contributing to peace building and social cohesion. Activities include farmer and pastoralist field schools and training for women’s groups on food processing, home gardening, healthy nutrition, and income generation. A pilot intervention aimed at improving food and nutrition security and reducing competition over natural resources has led to the introduction of low-space vertical gardening for the production of fodder and vegetables in IDP camps in North Darfur. This program has improved access to nutritious food and created new income opportunities, even when land and water are in short supply, and thus represents a solution adapted to the existing context. Welthungerhilfe also helps promote peaceful dialogue, coexistence, and reconciliation in North Darfur through community- based resolution mechanisms (CBRMs), which bring together pastoral and farmer communities of diverse ethnicities along migratory routes. CBRMs target youth at risk of becoming engaged in violence, as well as women, whose participation is crucial for mitigating and resolving disputes within and between communities. CBRMs offer workshops on migratory route awareness, rehabilitation of migratory routes, and sensitization of communities. Welthungerhilfe’s project has linked CBRMs with relevant government ministries, legal institutions, the Sudan Humanitarian Aid Commission, and security services, giving rural communities better access to legal avenues of conflict resolution and resources. Nonetheless, the situation remains volatile, with flare-ups of political instability and violence in addition to natural disasters and the pandemic. As recent political developments have destabilized the official judicial system, CBRMs have become more important than ever. Welthungerhilfe seeks to increase the inclusion of youth, women, and marginalized communities in the CBRMs. ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles4/article355_thumb.png" length="14714" type="image/png"/>
</entry>
<entry>
<title>
<![CDATA[ Partner Spotlight: Concern Worldwide in Haiti ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/case-studies/2021-haiti.html"/>
<id>urn:uuid:125db747-50b4-56d2-26ca-806c56b0a7e4</id>
<updated>2021-10-13T23:01:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles4/Haiti_thumb.png" alt="Partner Spotlight: Concern Worldwide in Haiti" /> <strong>The resilience of the Haitian people in the face of environmental, social, economic, and political instability is as extraordinary as the scale of challenges they face daily.</strong><br /> Box 2.2 PARTNER SPOTLIGHT: CONCERN WORLDWIDE IN HAITI On Saturday, August 14, 2021, Haiti was hit by a 7.2-magnitude earthquake. At the time of writing, the scale of the disaster was unclear, but early estimates of 1,300 dead, 5,700 injured, and more than 15,000 homes destroyed or damaged were all expected to rise. The resilience of the Haitian people in the face of environmental, social, economic, and political instability is as extraordinary as the scale of challenges they face daily. Though not at war, the country has suffered violence over many decades. In 2004, a UN peacekeeping mission was deployed there when, for the first time in history, a mandate was given authorizing the use of force—not to address an active conflict or enforce a peace agreement, but because the political and humanitarian crisis was a threat to international peace and security. That UN mission continued until 2017 and was followed by a smaller peacekeeping mission. Having worked in Haiti for more than 27 years, Concern Worldwide has learned a number of lessons about how best to help people build resilience to the shocks and stresses they are confronted with. Its resilience-building work has been focused especially on Haiti’s urban centers, where the majority of Haitians live. Growing urbanization in Haiti has led to a high concentration of the population in the metropolitan area of Port-au-Prince, where sprawling slums and high unemployment put enormous pressure on the area’s limited social infrastructure and basic services. Since long before the catastrophic 2010 earthquake, Haitians have suffered from degraded living conditions, limited educational opportunities, and poor economic prospects. In recent months the country’s sociopolitical and economic context has deteriorated further (President Jovenel Moïse was assassinated on July 7), leaving marginalized communities even more vulnerable to social and natural shocks. One of the areas where Concern Worldwide works is Cité Soleil, a marginalized and stigmatized commune in the Port-au-Prince area with a population of more than 265,000. Throughout 2021, tensions in the commune have been high. Fuel scarcity, traffic disruptions, and the closure of businesses and schools have harmed the livelihoods of the poorest households. According to the National Coordination for Food Security (CNSA), 46 percent of the population—4.4 million Haitians—are food insecure and in need of urgent humanitarian action. In Cité Soleil, at the time of writing, 55 percent of households are in a food crisis or food emergency (CNSA 2021). Against this backdrop, where hunger and conflict collide, Concern Worldwide’s integrated programming consists of a range of interventions that work holistically. Its approach prioritizes working with and through local facilitators and community health workers, and it places a strong emphasis on its relationships with local institutions. Its collaboration with the professional school Haiti Tec and the training center Centre Animation Paysanne et d’Action Communautaire (CAPAC), for example, has encouraged these institutions to make additional investments in vulnerable communities. As part of its adaptive approach, Concern Worldwide seeks to use technology to best effect, including using mobile phones to distribute vouchers or delivering radio broadcasts about good health and nutrition practices. Concern Worldwide’s integrated urban program is designed to meet people’s basic needs while building their capacity to meet their future needs. It provides people with the means to buy food while ensuring that markets have high-quality products from preapproved local suppliers. The team helps promote good health and nutritional practices so people can achieve both food security and nutrition security, which are especially critical at this time. Despite the challenging context and growing needs, Concern Worldwide—working in collaboration with partners and local communities—has had a positive impact on families living in Cité Soleil. Its programming has helped improve the food security of 3,000 of the commune’s most vulnerable and food-insecure households. Its interventions have increased households’ access to food, reduced the number of families resorting to negative coping strategies, and improved people’s nutrition behavior, including their consumption of fruits and vegetables and their dietary diversity. Concern’s food security programming has contributed to a rise in the food consumption score in the commune. Since the onset of the organization’s food security programming in Cité Soleil, the share of the population with an acceptable food consumption score has risen from 39 percent to 73 percent, and the share of the target population reporting poor food consumption has fallen from 25 percent to just 2.1 percent. In the face of the myriad challenges faced by the people of Haiti, it is critical that these gains be protected and built on over the months and years to come. ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles4/Haiti_thumb.png" length="10414" type="image/png"/>
</entry>
<entry>
<title>
<![CDATA[ Albania ]]>
</title>
<link type="text/html" href="https://www.globalhungerindex.org/albania.html"/>
<id>urn:uuid:d33df7e6-7958-6d46-271c-ab70f7fb3a86</id>
<updated>2021-10-13T21:04:00+00:00</updated>
<summary type="html">
<![CDATA[ <img style="margin:5px; float:left;" src="https://www.globalhungerindex.org/media/images/articles3/Albania_thumb.png" alt="Albania" /> <strong>With a score of 6.2 in the 2021 Global Hunger Index, Albania has a level of hunger that is low.</strong><br /> In the 2021 Global Hunger Index, Albania ranks 25th&nbsp;out of the 116 countries with sufficient data to calculate 2021 GHI scores. With a score of 6.2, Albania has a level of hunger that is low. ≤ 9.9 low 10.0–19.9 moderate 20.0–34.9 serious 35.0–49.9 alarming ≥ 50.0 extremely alarming &nbsp; Trend for Albania's Indicator Values &nbsp; Note: Data for GHI scores, child stunting, and child wasting are from 1998–2002 (2000), 2004–2008 (2006), 2010–2014 (2012), and 2016–2020 (2021). Data for undernourishment are from 2000–2002 (2000), 2005–2007 (2006), 2011–2013 (2012), and 2018–2020 (2021). Data for child mortality are from 2000, 2006, 2012, and 2019 (2021). See Appendix B for the formula for calculating GHI scores and Appendix C for the sources from which the data are compiled. &nbsp; Download/Print this Page .modulecountry ]]>
</summary>
<author>
<name>Ethical Sector</name>
</author>
<link rel="enclosure" href="https://www.globalhungerindex.org/media/images/articles3/Albania_thumb.png" length="18873" type="image/png"/>
</entry>
</feed>
"""


class MockResponse:
    status_code = 200
    elapsed = timedelta(seconds=1)
    headers = {}
    url = "https://www.globalhungerindex.org/atom.xml"
    text = mock_feed_text


@pytest.mark.parametrize(
    "configuration,expected_record_keys",
    [
        (
            {
                "feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"],
                "feed_entry_replication_key": "updated",
            },
            [
                "feed_url",
                "feed_title",
                "entry_id",
                "entry_updated",
                "entry_title",
                "entry_link",
            ],
        ),
        (
            {
                "feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"],
                "feed_fields": ["title"],
                "feed_entry_fields": ["id", "title", "link"],
                "feed_entry_replication_key": "updated",
            },
            [
                "feed_url",
                "feed_title",
                "entry_id",
                "entry_updated",
                "entry_title",
                "entry_link",
            ],
        ),
        (
            {
                "feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"],
                "feed_fields": ["title"],
                "feed_entry_fields": ["title"],
                "feed_entry_replication_key": "updated",
            },
            [
                "feed_url",
                "feed_title",
                "entry_id",
                "entry_updated",
                "entry_title",
            ],
        ),
        (
            {
                "feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"],
                "feed_fields": ["title", "link"],
                "feed_entry_fields": ["title", "author"],
                "feed_entry_replication_key": "updated",
            },
            [
                "feed_url",
                "feed_title",
                "feed_link",
                "entry_id",
                "entry_updated",
                "entry_title",
                "entry_author",
            ],
        ),
    ],
)
def test_parsing(monkeypatch, capfd, configuration, expected_record_keys):
    """Verifies the feed is parsed as expected"""

    def mock_get(*args, **kwargs):
        return MockResponse()

    test_tap: Tap = TapFeed(config=configuration)
    monkeypatch.setattr(test_tap.streams["feed"]._requests_session, "send", mock_get)

    test_tap.sync_all()
    out, err = capfd.readouterr()
    tap_records = get_parsed_records(out)
    assert len(tap_records) == 10
    for record in tap_records:
        print(record)
        assert record["type"] == "RECORD"
        assert record["stream"] == "feed"
        assert record["record"]["feed_url"] == MockResponse.url
        assert list(record["record"].keys()) == expected_record_keys


def get_parsed_records(tap_output: str) -> List[dict]:
    """Generates a list of the records from the stdout string provided"""
    return [
        json.loads(_ + "}") for _ in tap_output.split("}\n") if '"type": "RECORD' in _
    ]



"""Tests parsing of Atom 2.0 feed."""

from datetime import timedelta
import json
from typing import List

import pytest
from singer_sdk.tap_base import Tap

from tap_feed.tap import TapFeed


mock_feed_text = """
<rss xmlns:media="http://search.yahoo.com/mrss/" version="2.0">
<channel>
<title>Yahoo News - Latest News & Headlines</title>
<link>https://www.yahoo.com/news</link>
<description>The latest news and headlines from Yahoo! News. Get breaking news stories and in-depth coverage with videos and photos.</description>
<language>en-US</language>
<copyright>Copyright (c) 2021 Yahoo! Inc. All rights reserved</copyright>
<pubDate>Mon, 18 Oct 2021 21:51:11 -0400</pubDate>
<ttl>5</ttl>
<image>
<title>Yahoo News - Latest News & Headlines</title>
<link>https://www.yahoo.com/news</link>
<url>http://l.yimg.com/rz/d/yahoo_news_en-US_s_f_p_168x21_news.png</url>
</image>
<item>
<title>Bourbon producer signals intent to hire replacement workers</title>
<link>https://news.yahoo.com/bourbon-producer-signals-intent-hire-001239666.html</link>
<pubDate>2021-10-19T00:12:39Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">bourbon-producer-signals-intent-hire-001239666.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/bQxGjFE1TwiyDzqkFV23.A--~B/aD00NDgwO3c9NjcyMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/0440905775f152847bd84da925dfb9c0" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Kourtney Kardashian's massive engagement ring from Travis Barker is estimated to be worth $1 million</title>
<link>https://news.yahoo.com/kourtney-kardashians-massive-engagement-ring-164820709.html</link>
<pubDate>2021-10-18T16:48:20Z</pubDate>
<source url="https://www.insider.com/">INSIDER</source>
<guid isPermaLink="false">kourtney-kardashians-massive-engagement-ring-164820709.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/TEE8HwWjJmEU8Wi5dvBQTA--~B/aD0yMjQ5O3c9MzAwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/insider_articles_922/6b1bb0395ffbda90c1c3b24cb56da3c1" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Some Hooters servers on TikTok are saying new shorts are too short</title>
<link>https://news.yahoo.com/hooters-servers-tiktok-saying-shorts-164238995.html</link>
<pubDate>2021-10-17T15:46:31Z</pubDate>
<source url="https://www.nbcnews.com/">NBC News</source>
<guid isPermaLink="false">hooters-servers-tiktok-saying-shorts-164238995.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/6vXR7lxTgXTsUWvDby2_pA--~B/aD0xMjQ5O3c9MjQ5ODthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/nbc_news_122/e975b6d39d3ac2c4de2409bbe5fcc15c" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Train riders held up phones as woman was raped, police say</title>
<link>https://news.yahoo.com/train-riders-held-phones-woman-233116677.html</link>
<pubDate>2021-10-18T23:31:16Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">train-riders-held-phones-woman-233116677.html</guid>
<media:credit role="publishing company"/>
</item>
<item>
<title>Greta Thunberg 'Rickrolls' audience and busts out dance moves during climate concert</title>
<link>https://news.yahoo.com/greta-thunberg-rickrolls-audience-busts-234600985.html</link>
<pubDate>2021-10-18T23:46:00Z</pubDate>
<source url="https://www.washingtonexaminer.com/">Washington Examiner</source>
<guid isPermaLink="false">greta-thunberg-rickrolls-audience-busts-234600985.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/KWKJmcqQLcVrt1pIPbM7ag--~B/aD05NDA7dz0xNTQwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/washington_examiner_articles_265/23355ebe298e81c51a4c72cad15200ce" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>‘It was about to go down.’ Brawl at Panthers-Vikings game goes viral following tweet</title>
<link>https://news.yahoo.com/down-brawl-panthers-vikings-game-192316380.html</link>
<pubDate>2021-10-18T19:23:16Z</pubDate>
<source url="https://www.charlotteobserver.com/">Charlotte Observer</source>
<guid isPermaLink="false">down-brawl-panthers-vikings-game-192316380.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/Gc0DOSDl7ovZtHXs6HerrA--~B/aD0xNTU1O3c9MTE0MDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/charlotte_observer_mcclatchy_513/34d785311ddc07a6863a5564fa3506cb" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Woman denied organ transplant over refusal to get COVID-19 vaccine</title>
<link>https://news.yahoo.com/woman-denied-organ-transplant-over-170803360.html</link>
<pubDate>2021-10-18T17:08:03Z</pubDate>
<source url="https://www.cbsnews.com/videos/">CBS News Videos</source>
<guid isPermaLink="false">woman-denied-organ-transplant-over-170803360.html</guid>
<media:content height="86" url="https://s.yimg.com/hd/cp-video-transcode/prod/2021-10/18/616dadb89fbf3327ba0536a1/616dadb89fbf3327ba0536a2_o_U_v2.jpg" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Couple vocally opposed to vaccine die of COVID two weeks apart</title>
<link>https://news.yahoo.com/couple-vocally-opposed-vaccine-die-182558816.html</link>
<pubDate>2021-10-18T18:25:58Z</pubDate>
<source url="https://thegrio.com/">TheGrio</source>
<guid isPermaLink="false">couple-vocally-opposed-vaccine-die-182558816.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/RZXdmwDV_J.72t5VCKgG9g--~B/aD01NzY7dz0xMDI0O2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/thegrio_yahoo_articles_946/e9d356a1bea480073215b3ada130ee4c" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Bill and Melinda Gates seen together for first time since divorce at daughter's wedding</title>
<link>https://news.yahoo.com/bill-melinda-gates-seen-together-152115927.html</link>
<pubDate>2021-10-17T15:21:15Z</pubDate>
<source url="https://www.insider.com/">INSIDER</source>
<guid isPermaLink="false">bill-melinda-gates-seen-together-152115927.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/QZ47DBueI2Nx9WICzaZ04Q--~B/aD0yNjY1O3c9MzU1MzthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/insider_articles_922/0ade083549527ae52983fff7ba26a876" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>California congressional race could help tilt House control</title>
<link>https://news.yahoo.com/california-congressional-race-could-help-230845485.html</link>
<pubDate>2021-10-18T23:08:45Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">california-congressional-race-could-help-230845485.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/27yD_CWPMONZxKVbMQ7_4w--~B/aD0yNzE4O3c9Mzg0MDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/583c927ae9922089f54a2893aa947951" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>'America's Got Talent' shares update on Jonathan Goodwin after accident on set</title>
<link>https://news.yahoo.com/americas-got-talent-shares-jonathan-001011690.html</link>
<pubDate>2021-10-18T00:10:11Z</pubDate>
<source url="https://www.today.com/">TODAY</source>
<guid isPermaLink="false">americas-got-talent-shares-jonathan-001011690.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/1CMQfpU3u5yVBL45kR90Lg--~B/aD0xMjAwO3c9MjQwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/nbc_today_217/0c6b2dd480f2952e574987b61fc6984c" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>A California Construction Worker Asked a Speeding Motorist to Slow Down. He Was Shot Seven Times In Response.</title>
<link>https://news.yahoo.com/california-construction-worker-asked-speeding-123000097.html</link>
<pubDate>2021-10-18T12:30:00Z</pubDate>
<source url="https://atlantablackstar.com/">Atlanta Black Star</source>
<guid isPermaLink="false">california-construction-worker-asked-speeding-123000097.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/KJG_MsXNsb9_u.37UCqLrg--~B/aD0zMzg7dz02MDA7YXBwaWQ9eXRhY2h5b24-/https://media.zenfs.com/en/atlanta_black_star_articles_803/14f9be466b764c4a086a962c2c4fb2b8" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Freed murderer charged in Florida with slaying of single mom</title>
<link>https://news.yahoo.com/freed-murderer-charged-florida-slaying-195851469.html</link>
<pubDate>2021-10-17T19:58:51Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">freed-murderer-charged-florida-slaying-195851469.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/oDLRWX7drNRouaSxmIW07g--~B/aD0xMzg3O3c9MjAwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/9874f811f0df37fd9a6d54f02aea499b" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Fox News anchor John Roberts deleted a tweet that questioned vaccine efficacy in light of Colin Powell's death, clarifying that he is actually pro-vaccine</title>
<link>https://news.yahoo.com/fox-news-anchor-john-roberts-155319063.html</link>
<pubDate>2021-10-18T15:53:19Z</pubDate>
<source url="https://www.businessinsider.com/">Business Insider</source>
<guid isPermaLink="false">fox-news-anchor-john-roberts-155319063.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/Ofb8gK8x6fmUD9i8nSYJ3Q--~B/aD0xMzk4O3c9MTg2MzthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/business_insider_articles_888/e831b7b7b65d5e94f964a73ec8ea644d" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Dennis Prager Announces He Has COVID After Hugging ‘Thousands’ to Get It</title>
<link>https://news.yahoo.com/dennis-prager-announces-covid-hugging-212144657.html</link>
<pubDate>2021-10-18T21:21:44Z</pubDate>
<source url="http://www.thedailybeast.com">The Daily Beast</source>
<guid isPermaLink="false">dennis-prager-announces-covid-hugging-212144657.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/TXxCh_Wk4Qkkx6c84G95Gg--~B/aD02NTg7dz0xMTcwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/thedailybeast.com/487f6b9098e919b703fd498bfc44fa12" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Vaccines, masks? Japan puzzling over sudden virus success</title>
<link>https://news.yahoo.com/vaccines-masks-japan-puzzling-over-034036438.html</link>
<pubDate>2021-10-18T03:40:36Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">vaccines-masks-japan-puzzling-over-034036438.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/ZqA52aMGdS3qprPBbGNQcw--~B/aD0zMzMzO3c9NTAwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/30066b303bf37f3629a8a41e5e45ef56" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>'Eternals' star Salma Hayek says she had 'problems with the script' and 'got into a serious fight' with director Chloé Zhao, but they found 'middle ground'</title>
<link>https://news.yahoo.com/eternals-star-salma-hayek-says-173151941.html</link>
<pubDate>2021-10-18T17:31:51Z</pubDate>
<source url="https://www.insider.com/">INSIDER</source>
<guid isPermaLink="false">eternals-star-salma-hayek-says-173151941.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/cZGes9xdBWLFHuhuBig91Q--~B/aD0xNzE1O3c9MjI4NzthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/insider_articles_922/7740f63b4b412d5735f2e636d95f2e4b" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Eric Stonestreet says it's 'ridiculous' that fans assume he, fiancée have large age gap: 'She looks fantastic'</title>
<link>https://news.yahoo.com/eric-stonestreet-says-apos-apos-213249910.html</link>
<pubDate>2021-10-18T21:32:49Z</pubDate>
<source url="https://www.foxnews.com/">Fox News</source>
<guid isPermaLink="false">eric-stonestreet-says-apos-apos-213249910.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/eNWAvThEdtIpAZ5YjUYdWg--~B/aD03MjA7dz0xMjgwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/fox_news_text_979/b7c5345f1346cc05d1220e909747273c" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Oregon illegal pot grows: More calls to send National Guard</title>
<link>https://news.yahoo.com/oregon-illegal-pot-grows-more-204121840.html</link>
<pubDate>2021-10-18T20:41:21Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">oregon-illegal-pot-grows-more-204121840.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/VzvoC0LW9LgECK.RVtzcXw--~B/aD0xNjYzO3c9MjQ5NDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/f1903c6d5184cf27831d197b0fd4c0b3" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>200 painted, naked models descend on Dead Sea for photo shoot</title>
<link>https://news.yahoo.com/200-painted-naked-models-descend-165700229.html</link>
<pubDate>2021-10-18T16:57:00Z</pubDate>
<source url="https://www.washingtonexaminer.com/">Washington Examiner</source>
<guid isPermaLink="false">200-painted-naked-models-descend-165700229.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/altGL3m0Y38XudH93jSsrQ--~B/aD05NDA7dz0xNTQwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/washington_examiner_articles_265/435bad37036b277d5ea42276c86189b6" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Trump answered questions for 4 hours in a deposition for a lawsuit alleging his bodyguards beat up protesters outside Trump Tower</title>
<link>https://news.yahoo.com/trump-answered-questions-4-hours-212359653.html</link>
<pubDate>2021-10-18T21:23:59Z</pubDate>
<source url="https://www.businessinsider.com/">Business Insider</source>
<guid isPermaLink="false">trump-answered-questions-4-hours-212359653.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/ZMN2MDisO8RC0iz7pqVvTQ--~B/aD0xMTM3O3c9MTUxNjthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/business_insider_articles_888/07c09df6d9eb34f11bed2a24cf6a0111" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Louisiana gators thrive, so farmers' return quota may drop</title>
<link>https://news.yahoo.com/louisiana-gators-thrive-farmers-return-144731641.html</link>
<pubDate>2021-10-17T14:47:31Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">louisiana-gators-thrive-farmers-return-144731641.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/9qbq6w7xT6JG76JmGkfI1g--~B/aD0yNDc0O3c9MzcxMTthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/d3c366828b6dcd7dd9243819d1e0aa94" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Ted Cruz blasted Australia’s COVID rules. A top Australian official didn’t hold back</title>
<link>https://news.yahoo.com/ted-cruz-blasted-australia-covid-135920141.html</link>
<pubDate>2021-10-18T13:59:20Z</pubDate>
<source url="https://www.star-telegram.com/">Fort Worth Star-Telegram</source>
<guid isPermaLink="false">ted-cruz-blasted-australia-covid-135920141.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/cbY5Lyh4dImFVV8XWQIhMQ--~B/aD02NDE7dz0xMTQwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/fort_worth_star_telegram_mcclatchy_952/7ada7b1467e3a9b7f9125aacd355732f" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Man charged with assault after prosecutors say he grabbed another parent at a Minnesota school board meeting about masking</title>
<link>https://news.yahoo.com/man-charged-assault-prosecutors-grabbed-220109239.html</link>
<pubDate>2021-10-18T22:01:09Z</pubDate>
<source url="https://www.insider.com/">INSIDER</source>
<guid isPermaLink="false">man-charged-assault-prosecutors-grabbed-220109239.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/yCS8PKcRYy5YlLjDL7BI2A--~B/aD00NjA7dz02MTM7YXBwaWQ9eXRhY2h5b24-/https://media.zenfs.com/en/insider_articles_922/1650d4359777548ee766c2310e835ef3" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Japan PM says Fukushima wastewater release can't be delayed</title>
<link>https://news.yahoo.com/japan-pm-says-fukushima-wastewater-125248800.html</link>
<pubDate>2021-10-17T12:52:48Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">japan-pm-says-fukushima-wastewater-125248800.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/rtyjjFU2VFTuYkcvqxJbkw--~B/aD0zNjQ4O3c9NTQ3MjthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/fb07bd0b7c85b2bcef4bc867b2337668" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>She thought she won $300 — but her lottery ticket was worth much more in South Carolina</title>
<link>https://news.yahoo.com/she-thought-she-won-300-144711801.html</link>
<pubDate>2021-10-18T14:47:11Z</pubDate>
<source url="https://www.thestate.com/">The State</source>
<guid isPermaLink="false">she-thought-she-won-300-144711801.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/28C1HgJiXXx6wrCtFf0RKg--~B/aD03NTk7dz0xMTQwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/the_state_mcclatchy_264/5b1799ef12d34403009b4f5602a6248d" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Brian Laundrie lookalike claims US Marshals burst into hotel room</title>
<link>https://news.yahoo.com/brian-laundrie-lookalike-claims-us-214200021.html</link>
<pubDate>2021-10-18T21:42:00Z</pubDate>
<source url="https://www.washingtonexaminer.com/">Washington Examiner</source>
<guid isPermaLink="false">brian-laundrie-lookalike-claims-us-214200021.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/T1koH.vRvjOAuCCOJITYdw--~B/aD05NDA7dz0xNTQwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/washington_examiner_articles_265/0527dcb4a31d7242503aa1fc5b45ce7e" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Philippine forces kill 4 suspected Chinese drug dealers</title>
<link>https://news.yahoo.com/philippine-forces-kill-4-suspected-141403231.html</link>
<pubDate>2021-10-18T14:14:03Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">philippine-forces-kill-4-suspected-141403231.html</guid>
<media:content height="86" url="https://s.yimg.com/os/creatr-uploaded-images/2021-10/b96d2a40-3032-11ec-affc-8609c7aa0cf2" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Jordan Klepper Exposes MAGA Morons Who Still Think ‘Trump Won’</title>
<link>https://news.yahoo.com/jordan-klepper-exposes-maga-morons-010010286.html</link>
<pubDate>2021-10-19T01:00:10Z</pubDate>
<source url="http://www.thedailybeast.com">The Daily Beast</source>
<guid isPermaLink="false">jordan-klepper-exposes-maga-morons-010010286.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/VdarZRwy_nTBNQVg86QxdQ--~B/aD02NTg7dz0xMTcwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/thedailybeast.com/4dba2aa912b7ad864de728d63183758c" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Woman's body recovered from mountain after she sent message for help</title>
<link>https://news.yahoo.com/climbers-body-recovered-mountain-she-125113389.html</link>
<pubDate>2021-10-18T12:51:00Z</pubDate>
<source url="https://www.cbsnews.com/">CBS News</source>
<guid isPermaLink="false">climbers-body-recovered-mountain-she-125113389.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/pYczVCCOcp11IkYZGjudwA--~B/aD02Njc7dz0xMDE4O2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/cbs_news_897/29ac5daa425908dcefa67bbab7920892" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Coast Guard: 1,200-foot ship dragged California oil pipeline</title>
<link>https://news.yahoo.com/coast-guard-1-200-foot-174350385.html</link>
<pubDate>2021-10-17T17:43:50Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">coast-guard-1-200-foot-174350385.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/LqC_25_eISFO.Iqf7BbLZA--~B/aD0yMDAwO3c9MzAwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/c93491e5d1909c4422311dc2e9aa562d" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Brian Laundrie's father was captured removing a yellow protest sign about his daughter from his front yard, report says</title>
<link>https://news.yahoo.com/brian-laundries-father-captured-removing-203330639.html</link>
<pubDate>2021-10-17T20:33:30Z</pubDate>
<source url="https://www.insider.com/">INSIDER</source>
<guid isPermaLink="false">brian-laundries-father-captured-removing-203330639.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/hS7wyUvd4ZokwRXX9VssgQ--~B/aD0xMTA0O3c9MTQ3MzthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/insider_articles_922/10dff734740fcb31cfd070f0ea4c8f94" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Untaming a river: The stakes behind America’s largest dam removal</title>
<link>https://news.yahoo.com/untaming-river-stakes-behind-america-085100296.html</link>
<pubDate>2021-10-18T08:51:00Z</pubDate>
<source url="http://www.csmonitor.com/">Christian Science Monitor</source>
<guid isPermaLink="false">untaming-river-stakes-behind-america-085100296.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/H228Vj0U7RnV5uwPH73BRg--~B/aD02MDA7dz05MDA7YXBwaWQ9eXRhY2h5b24-/https://media.zenfs.com/en/csmonitor.com/b6bee7d951a29248539bd06074fc870c" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>U.S. Supreme Court again protects police accused of excessive force</title>
<link>https://news.yahoo.com/u-supreme-court-again-protects-181825164.html</link>
<pubDate>2021-10-18T18:18:25Z</pubDate>
<source url="https://www.yahoo.com/news">Yahoo News Video</source>
<guid isPermaLink="false">u-supreme-court-again-protects-181825164.html</guid>
<media:content height="86" url="https://s.yimg.com/hd/cp-video-transcode/prod/2021-10/18/616dba72309e6d28efcee16c/616dba72309e6d28efcee16d_o_U_v2.jpg" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Progressive activists demand Rahm Emanuel nomination removal</title>
<link>https://news.yahoo.com/progressive-activists-demand-rahm-emanuel-003332340.html</link>
<pubDate>2021-10-19T00:33:32Z</pubDate>
<source url="https://www.axios.com/">Axios</source>
<guid isPermaLink="false">progressive-activists-demand-rahm-emanuel-003332340.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/jBdnhNwuRyw9ODlwNNth_A--~B/aD03MjA7dz0xMjgwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/axios_articles_623/a799a4907ec9a05cdf1a1d7c048450f6" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Twitter Users Snarkily Respond To Trump's Lawsuit Against Jan. 6 Documents</title>
<link>https://news.yahoo.com/twitter-users-snarkily-respond-trumps-233821581.html</link>
<pubDate>2021-10-18T23:38:21Z</pubDate>
<source url="https://www.huffpost.com">HuffPost</source>
<guid isPermaLink="false">twitter-users-snarkily-respond-trumps-233821581.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/IsMseGCnQQuvxK1p5f_bQA--~B/aD02MDA7dz0xMjAwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/the_huffington_post_584/38fd819ef6dccb38a69fa2de1eb69086" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Gambian Toufah Jallow tells of surviving rape by dictator</title>
<link>https://news.yahoo.com/gambian-toufah-jallow-tells-surviving-082807483.html</link>
<pubDate>2021-10-17T08:28:07Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">gambian-toufah-jallow-tells-surviving-082807483.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/el_vpQQ820Hwglgrlz3wUg--~B/aD0yNDQ3O3c9MTYzMjthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/2a06435c64f90793db5b9c49245f61d1" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>The Suns gave $133 million to 2 role players in 2 days after deciding not to give their No. 1 pick a max contract</title>
<link>https://news.yahoo.com/suns-gave-133-million-2-230339362.html</link>
<pubDate>2021-10-18T23:03:39Z</pubDate>
<source url="https://www.insider.com/">INSIDER</source>
<guid isPermaLink="false">suns-gave-133-million-2-230339362.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/iSdH6Jjs5G1kLF7Gn.p3kQ--~B/aD0xMTY3O3c9MTU1NzthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/insider_articles_922/fa0a21ecd9d6bd68d82c2bc295d19911" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>VIDEO: Deadly floods and landslides sweep away homes in southern India</title>
<link>https://news.yahoo.com/video-deadly-floods-landslides-sweep-211138272.html</link>
<pubDate>2021-10-18T21:11:38Z</pubDate>
<source url="https://www.insider.com/">INSIDER Video</source>
<guid isPermaLink="false">video-deadly-floods-landslides-sweep-211138272.html</guid>
<media:content height="86" url="https://s.yimg.com/hd/cp-video-transcode/prod/2021-10/18/616de44888b0ef282fb698f2/616de44888b0ef282fb698f3_o_U_v2.jpg" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>South Dakota lawmakers summon key figures for Noem inquiry</title>
<link>https://news.yahoo.com/south-dakota-lawmakers-summon-key-175834782.html</link>
<pubDate>2021-10-18T17:58:34Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">south-dakota-lawmakers-summon-key-175834782.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/OiBg8055rqNlCrRI5ppfXg--~B/aD01NjUxO3c9ODQ3NjthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/cdac5ecb2a7dc055078af5769efe6f1a" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Food Network star Anne Burrell gets married: 'Luckiest girl in the world'</title>
<link>https://news.yahoo.com/food-network-star-anne-burrell-150131304.html</link>
<pubDate>2021-10-17T15:01:31Z</pubDate>
<source url="https://www.today.com/">TODAY</source>
<guid isPermaLink="false">food-network-star-anne-burrell-150131304.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/qjG2VogCr.TgaXdEUOUHNQ--~B/aD0xMjAwO3c9MjQwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/nbc_today_217/59a6816d15ece06faf4dc49b48d4eead" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Billionaire ex-Walmart exec says the first 'settlers' of his planned $400 billion city 'Telosa' will likely be selected through applications - and they could move in by 2030</title>
<link>https://news.yahoo.com/billionaire-ex-walmart-exec-says-161647946.html</link>
<pubDate>2021-10-18T16:16:47Z</pubDate>
<source url="https://www.businessinsider.com/">Business Insider</source>
<guid isPermaLink="false">billionaire-ex-walmart-exec-says-161647946.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/hZkVTjRVuUHnHEYJ0ePJ_Q--~B/aD0xNDA1O3c9MTg3NDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/business_insider_articles_888/54e8469678fd23fcf319239e614006f3" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Rare Sumatran tiger found dead in animal trap in Indonesia</title>
<link>https://news.yahoo.com/rare-sumatran-tiger-found-dead-103208001.html</link>
<pubDate>2021-10-18T10:32:08Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">rare-sumatran-tiger-found-dead-103208001.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/nG.nKRCyOqgVch.HikPw0g--~B/aD00MDAxO3c9NjAwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/31097eab8bdc091fb9e59ffac0b93e08" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Trump says he wouldn't need vaccine mandates and would convince people: 'I would sell it'</title>
<link>https://news.yahoo.com/trump-says-wouldnt-vaccine-mandates-200900849.html</link>
<pubDate>2021-10-18T20:09:00Z</pubDate>
<source url="https://www.washingtonexaminer.com/">Washington Examiner</source>
<guid isPermaLink="false">trump-says-wouldnt-vaccine-mandates-200900849.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/.DMVfwKi_LorccSplMjzdg--~B/aD05NDA7dz0xNTQwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/washington_examiner_articles_265/09e613d0c2dd2f9fb72e4e3cfdb77fec" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Paul McCartney opens up about the real woman who inspired 'Eleanor Rigby' and says she 'enriched my soul'</title>
<link>https://news.yahoo.com/paul-mccartney-opens-real-woman-175306632.html</link>
<pubDate>2021-10-18T17:53:06Z</pubDate>
<source url="https://www.insider.com/">INSIDER</source>
<guid isPermaLink="false">paul-mccartney-opens-real-woman-175306632.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/76_7GS0xvcxUzBYefvXkoQ--~B/aD05MDA7dz0xMjAwO2FwcGlkPXl0YWNoeW9u/https://media.zenfs.com/en/insider_articles_922/b9b7a576535d5206b008e16fa5229b75" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Analysis: NFL loaded with mediocre teams 6 weeks into season</title>
<link>https://news.yahoo.com/nfl-loaded-mediocre-poor-teams-071427781.html</link>
<pubDate>2021-10-18T07:14:27Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">nfl-loaded-mediocre-poor-teams-071427781.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/9ZdgmW7NUj7N32fjoWK0Dw--~B/aD00MDAwO3c9NjAwMDthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/ap.org/6d9bdf234dc2db56e85ba723c1489592" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Three dead, including suspect, after man cuts throat of Arkansas officer</title>
<link>https://news.yahoo.com/three-dead-including-suspect-man-223400267.html</link>
<pubDate>2021-10-17T22:34:00Z</pubDate>
<source url="https://www.nbcnews.com/">NBC News</source>
<guid isPermaLink="false">three-dead-including-suspect-man-223400267.html</guid>
<media:credit role="publishing company"/>
</item>
<item>
<title>A Florida school says vaccinated students must stay home for 30 days after each shot, citing a false claim that they'll infect others</title>
<link>https://news.yahoo.com/florida-school-says-students-vaxxed-181722021.html</link>
<pubDate>2021-10-18T17:33:29Z</pubDate>
<source url="https://www.businessinsider.com/">Business Insider</source>
<guid isPermaLink="false">florida-school-says-students-vaxxed-181722021.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/75sQ44WrVwSJSlPW5xJ43w--~B/aD0xMzk4O3c9MTg2OTthcHBpZD15dGFjaHlvbg--/https://media.zenfs.com/en/business_insider_articles_888/311345bdb83ece7c504fd34567562cbf" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Judge denies request to block lethal injection in Alabama</title>
<link>https://news.yahoo.com/judge-denies-request-block-lethal-181058527.html</link>
<pubDate>2021-10-18T18:10:58Z</pubDate>
<source url="http://www.ap.org/">Associated Press</source>
<guid isPermaLink="false">judge-denies-request-block-lethal-181058527.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/gcek8z5gLf.nyagaN_6udg--~B/aD01MzU7dz00NTg7YXBwaWQ9eXRhY2h5b24-/https://media.zenfs.com/en/ap.org/a11d53d22dde44d5b3e6ed3e8529d24d" width="130"/>
<media:credit role="publishing company"/>
</item>
<item>
<title>Republican resistance to Democrats' anti-gerrymandering efforts could lead to a GOP 'takeover'</title>
<link>https://news.yahoo.com/republican-resistance-democrats-anti-gerrymandering-214800354.html</link>
<pubDate>2021-10-18T21:48:00Z</pubDate>
<source url="https://theweek.com/">The Week</source>
<guid isPermaLink="false">republican-resistance-democrats-anti-gerrymandering-214800354.html</guid>
<media:content height="86" url="https://s.yimg.com/uu/api/res/1.2/A881V3VyNNv9IO_zGKcV6g--~B/aD02MTg7dz04NDU7YXBwaWQ9eXRhY2h5b24-/https://media.zenfs.com/en/the_week_574/65ee2726a705662d6433a797c90986e4" width="130"/>
<media:credit role="publishing company"/>
</item>
</channel>
</rss>
"""


class MockResponse:
    status_code = 200
    elapsed = timedelta(seconds=1)
    headers = {}
    url = "https://www.yahoo.com/news/rss"
    text = mock_feed_text


@pytest.mark.parametrize(
    "configuration,expected_record_keys",
    [
        (
            {"feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"]},
            [
                "feed_url",
                "feed_title",
                "entry_id",
                "entry_published",
                "entry_title",
                "entry_link",
            ],
        ),
        (
            {
                "feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"],
                "feed_fields": ["title"],
                "feed_entry_fields": ["id", "title", "link"],
            },
            [
                "feed_url",
                "feed_title",
                "entry_id",
                "entry_published",
                "entry_title",
                "entry_link",
            ],
        ),
        (
            {
                "feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"],
                "feed_fields": ["title"],
                "feed_entry_fields": ["title"],
            },
            [
                "feed_url",
                "feed_title",
                "entry_id",
                "entry_published",
                "entry_title",
            ],
        ),
        (
            {
                "feed_urls": ["http://Ill-be-replaced-through-monkeypatch.com"],
                "feed_fields": ["title", "language"],
                "feed_entry_fields": ["title", "guid", "source"],
            },
            [
                "feed_url",
                "feed_title",
                "feed_language",
                "entry_id",
                "entry_published",
                "entry_title",
                "entry_guid",
                "entry_source",
            ],
        ),
    ],
)
def test_parsing(monkeypatch, capfd, configuration, expected_record_keys):
    """Verifies the feed is parsed as expected"""

    def mock_get(*args, **kwargs):
        return MockResponse()

    test_tap: Tap = TapFeed(config=configuration)
    monkeypatch.setattr(test_tap.streams["feed"]._requests_session, "send", mock_get)

    test_tap.sync_all()
    out, err = capfd.readouterr()
    tap_records = get_parsed_records(out)
    assert len(tap_records) == 50
    for record in tap_records:
        assert record["type"] == "RECORD"
        assert record["stream"] == "feed"
        assert record["record"]["feed_url"] == MockResponse.url
        assert list(record["record"].keys()) == expected_record_keys


def get_parsed_records(tap_output: str) -> List[dict]:
    """Generates a list of the records from the stdout string provided"""
    return [
        json.loads(_ + "}") for _ in tap_output.split("}\n") if '"type": "RECORD' in _
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tap_feed', 'tap_feed.tests', 'tap_feed.tests.parser']

package_data = \
{'': ['*']}

install_requires = \
['feedparser>=6.0.8,<7.0.0',
 'requests>=2.25.1,<3.0.0',
 'singer-sdk>=0.3.11,<0.4.0']

entry_points = \
{'console_scripts': ['tap-feed = tap_feed.tap:TapFeed.cli']}

setup_kwargs = {
    'name': 'tap-feed',
    'version': '1.1.0',
    'description': 'A Singer tap for RSS and Atom feeds built with the Meltano SDK for Singer Taps.',
    'long_description': '[![Tests](https://github.com/jawats/tap-feed/actions/workflows/tests.yml/badge.svg)](https://github.com/jawats/tap-feed/actions/workflows/tests.yml)\n[![Code Style](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black) \n\n# tap-feed\n\n`tap-feed` is a Singer tap for various feeds and was built with the [Meltano Tap SDK](https://sdk.meltano.com) for Singer Taps.\nThis tap can be used with RSS and Atom based feeds thanks to [feedparser](https://feedparser.readthedocs.io/en/latest/index.html).\n\n## Installation\n\nTo install this tap, simply run the following command in your terminal:\n\n```bash\npipx install tap-feed\n```\n\nOr if you don\'t want to use pipx:\n\n```bash\npip3 install tap-feed\n```\n\n## Configuration\n\n### Accepted Config Options\n\n| Property                   | Type             | Required? | Description                                                                |\n| ---                        | ---              | ---       | ---                                                                        |\n| feed_urls                  | String           | Yes       | A list of one or more feed paths                                           |\n| feed_fields                 | Array of Strings | Yes       | A list of feed level data fields to capture                                 |\n| feed_entry_fields           | Array of Strings | Yes       | A list of entry level data fields to capture                                |\n| feed_entry_replication_key | String           | Yes       | The field used to determine new records, typically \'published\' or \'updated\' |\n| start_date                 | Date Time        | No        | The earliest record date to sync                                           |\n\nA full list of supported settings and capabilities for this\ntap is available by running:\n\n```bash\ntap-feed --about\n```\n\n### Example Config File\n```json\n{\n  "feed_urls": ["http://feeds.feedburner.com/PythonSoftwareFoundationNews", "https://talkpython.fm/episodes/rss"],\n  "feed_fields": ["title", "subtitle"],\n  "feed_entry_fields": ["id", "title", "link"],\n  "feed_entry_replication_key": "published"\n}\n```\n\n## Usage\n\nYou can easily run `tap-feed` by itself or in a pipeline using [Meltano](https://meltano.com/).\n\n### Executing the Tap Directly\n\n- First create a file containing the configuration in a json format, e.g., config.json\n- Use the config file to create a catalog file and then invoke the tap\n\n```bash\ntap-feed --config config.json --discover > catalog.json\ntap-feed --config config.json --catalog catalog.json\n```\n\n',
    'author': 'Jon Watson',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jawats/tap-feed',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<3.10',
}


setup(**setup_kwargs)



"""Stream type classes for tap-feed."""

from typing import List, Iterable, Optional

from dateutil import parser as datetime_parser
import feedparser  # type: ignore
import requests
from singer_sdk import typing as th  # JSON Schema typing helpers
from singer_sdk.streams import RESTStream


class FeedStream(RESTStream):
    """FeedStream class."""

    url_base = ""
    name = "feed"
    path = ""
    primary_keys = ["entry_id"]
    replication_key = ""

    @property
    def schema(self) -> dict:
        """Create a dynamic schema based on the tap configuration provided."""
        feed_properties: list = [
            th.Property("feed_url", th.StringType),
        ]
        feed_properties.extend(
            [
                th.Property(f"feed_{name}", th.StringType)
                for name in self.config["feed_fields"]
            ]
        )
        feed_properties.extend(
            [
                th.Property("entry_id", th.StringType),
                th.Property(FeedStream.replication_key, th.DateTimeType),
            ]
        )
        feed_properties.extend(
            [
                th.Property(f"entry_{name}", th.StringType)
                for name in self.config["feed_entry_fields"]
                if name not in ["id", "published", "updated"]
            ]
        )
        return th.PropertiesList(*feed_properties).to_dict()

    @property
    def partitions(self) -> Optional[List[dict]]:
        """Return a list of partition key dicts for the feed URLs."""
        return [{"feed_url": url} for url in self.config["feed_urls"]]

    def get_url(self, context: Optional[dict]) -> str:
        """Get the stream entity URL for the current partition."""
        partition = context or {}
        return partition["feed_url"]

    def parse_response(self, response: requests.Response) -> Iterable[dict]:
        """Parse the response and return an iterator of result rows."""
        feed = feedparser.parse(response.text)
        rep_key = FeedStream.replication_key[6:]
        entries = sorted(
            feed["entries"], key=lambda d: datetime_parser.parse(d[rep_key])
        )
        feed_url = response.url
        bookmark_timestamp = self.get_starting_timestamp({"feed_url": feed_url})
        for entry in entries:
            published_str = entry.get(rep_key)
            if published_str is None:
                continue
            published_date = datetime_parser.parse(published_str)
            if bookmark_timestamp is None or published_date > bookmark_timestamp:
                record = {"feed_url": feed_url}
                record.update(
                    {
                        f"feed_{name}": feed["feed"].get(name)
                        for name in self.config["feed_fields"]
                    }
                )
                record.update(
                    {
                        "entry_id": entry["id"],
                        FeedStream.replication_key: str(published_date),
                    }
                )
                record.update(
                    {
                        f"entry_{name}": entry.get(name)
                        for name in self.config["feed_entry_fields"]
                        if name not in ["id", "published", "updated"]
                    }
                )
                yield record
